format binary as ""
use32
org	0x0
db	'MENUET01'
dd	1, START, I_END, mem, mem, 0, 0


; Socket buffer size
SNDBUFFER		= 4096
RCVBUFFER		= 4096

; Network consts
STATUS_DISCONNECTED	= 0
STATUS_RESOLVING	= 1
STATUS_CONNECTING	= 2
STATUS_CONNECTED	= 3


; Common libs
include 'macros.inc'
include 'proc32.inc'
include 'dll.inc'
include 'box_lib.mac'
; Custom libs
include 'rxl_node.inc'
include 'rxl_attr.inc'
include 'rxl_read.inc'
include 'rxl_write.inc'
; XMPP libs
include 'login.asm'
include 'net.asm'
include 'common.asm'
include 'xmpp_attr.asm'
include 'xmpp_const.asm'
include 'xmpp_stream.asm'
include 'xmpp_xml.asm'
include 'debug.asm'


;<<<<<<<<<<<<<<<<<<<
;
;	CODE BLOCK
;
;<<<<<<<<<<<<<<<<<<<

START:
	; Initialize heap
	mcall	68, 11

	; Load libraries
	stdcall dll.Load, @IMPORT
	test	eax, eax
	jnz	close_progr

	mcall	40, EVM_REDRAW + EVM_KEY + EVM_BUTTON + EVM_MOUSE

login_win_redraw:
	call	login_win_draw

wait_login:
	; Wait for action
	mcall	10
	; Check if need to redraw
	dec	eax
	jz	login_win_redraw
	; Key in buffer
	dec	eax
	jz	login_key
	; Check if any button was pressed
	dec	eax
	jz	login_btn_action

	invoke	edit_box_mouse, login_box
	invoke	edit_box_mouse, pass_box
	invoke	edit_box_mouse, serv_box
	invoke	edit_box_mouse, dbg_box

	; Get network event
	call	network_event		; ???

	; If nothing happened
	jmp	wait_login

get_sys_colours:
	; Push all regs to stack saving them
	pusha
	; fn 48 - sys colours
	mov	eax, 48
	; fn 48.ebx - get
	mov	ebx, 3
	; fn 48.3.ecx - pointer to var sys_colours
	mov	ecx, sys_colours
	; fn 48.3.edx - return 40 bytes
	mov	edx, 10*4
	mcall
	; Pop all regs
	popa
	; Return
	ret

; Button was pressed
login_btn_action:
	; Get button number
	mcall	17
	; If id = 1 (red cross) - close program
	cmp	ah, 1
	je	close_progr
	; If id = 2 - login button
	cmp	ah, 2
	je	login_submit
	; Wait for next input
	jmp	wait_login

; Key was pressed
login_key:
	; Receive pressed key into eax
	mcall	2
	; Add value into loginbox if it was editbox key
	invoke	edit_box_key, login_box
	invoke	edit_box_key, pass_box
	invoke	edit_box_key, serv_box
	invoke	edit_box_key, dbg_box
	; Wait for next input
	jmp	wait_login

; Login action
login_submit:
	construct_stream	x_stream, login_text, serv_text
	pushStreamStruc 	x_stream
	stdcall strcpy, xc_xml_start, snd_buffer
	createXmlOpen		xc_stream, xc_stream, edi

dns_res:
; RESOLVE DNS
	stdcall resolve_dns, x_stream.to
	test	eax, eax	; Test for error
	jnz	wait_login	; Jump if error

; GET IP AND PORT
	stdcall convert_IP_dec, ebx	; If no error - get decimal IP
	mov	[sockaddr1.port], 0x6614
; OPEN SOCKET
	call	open_socket
send_login:
	stdcall strlen, snd_buffer
	mov	[msg_len], ecx
	mcall	send, [socketnum], snd_buffer, [msg_len], 0
	jmp wait_login





network_event:
	mov	eax, [status]
	dec	eax
	; STATUS_DISCONNECTED
	js	.nothing
	; STATUS_RESOLVING
	jz	.nothing
	; STATUS_CONNECTING
	dec	eax
	jz	.nothing
	; STATUS_CONNECTED
	jmp	.connected

 .nothing:
	ret

 .connecting:

 .connected:
	call	socket_receive
	ret


socket_receive:
	mcall	recv, [socketnum], rcv_buffer, RCVBUFFER, MSG_DONTWAIT
;tls_begin:
;	 push	 0
;	 push	 xc_starttls_ns
;	 push	 xc_ns
;	 createXmlOpen	 xc_starttls,'',snd_buffer
;	 createXmlClose  xc_starttls,'',eax
;	 stdcall strlen,snd_buffer
;	 mov	 [msg_len],ecx
;tls_send:
;	 mcall	 send,[socketnum],snd_buffer,[msg_len],0



;new_stream:
;	 construct_stream	 x_stream,login_text,serv_text
;	 pushStreamStruc	 x_stream
;	 stdcall		 strcpy,xc_xml_start,snd_buffer
;	 createXmlOpen		 xc_stream,xc_stream,edi
;	 stdcall strlen,snd_buffer
;	 mov	 [msg_len],ecx
;new_stream_send:
;	 mcall	 send,[socketnum],snd_buffer,[msg_len],0



auth:
	push	0
	push	xc_auth_plain
	push	xc_auth_mech
	push	xc_auth_sasl
	push	xc_ns
	createXmlOpen	xc_auth,'',snd_buffer
	stdcall strlen,test_auth
	stdcall base64_encode,test_auth,ecx,eax
	stdcall strlen,snd_buffer
	add	ecx,snd_buffer
	createXmlClose	xc_auth,'',ecx
	stdcall strlen,snd_buffer
	mov	[msg_len], ecx
send_auth:
	mcall	send, [socketnum], snd_buffer, [msg_len], 0
	jmp	wait_login

free_alloc_memory:
	push	esi
	call	[freeaddrinfo]
	ret


close_progr:
	mcall	-1


;<<<<<<<<<<<<<<<<<<<
;
;	DATA BLOCK
;
;<<<<<<<<<<<<<<<<<<<

; Initialised data

;<<<<<<<<<<<<<<<<<<<<<<<<<<
test_auth	db '\0test\0test',0
;<<<<<<<<<<<<<<<<<<<<<<<<<<

sockaddr1:
	dw	AF_INET4
	.port	dw 0
	.ip	dd 0
	rb	10		; Address buffer

status		dd STATUS_DISCONNECTED
msg_len 	dd 0

align 4
@IMPORT:

library network, 'network.obj',\
	box_lib, 'box_lib.obj'

import	network, \
	getaddrinfo,	'getaddrinfo',\
	freeaddrinfo,	'freeaddrinfo',\
	inet_ntoa,	'inet_ntoa'

import	box_lib, \
	edit_box_draw,	'edit_box',\
	edit_box_mouse, 'edit_box_mouse',\
	edit_box_key,	'edit_box_key'


; End of program
I_END:

; Uninitialised data:

ip_ptr		dd ?
socketnum	dd ?

x_stream	xmpp_stream

; Sys colours table
sys_colours	system_colors

; Receive buffer
snd_buffer	rb SNDBUFFER
rcv_buffer	rb RCVBUFFER
tmp_buffer	rb 1024
; Additional buffer
		rb 2048
mem:
